//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

extern "C"
{
/* ==================================================================== */
#include "api_scilab.h"
#include "Scierror.h"
#include "version.h"
#if SCI_VERSION_MAJOR < 6
// Scilab 5
#include "MALLOC.h"
#else
// Scilab 6
#include "sci_malloc.h"
#endif
#include <localization.h>
#include "CLx_gw_util.h"
#include "clx_msis00.h"

/* ==================================================================== */
#if SCI_VERSION_MAJOR  < 6
// Scilab 5
int CLx_gw_msis00(char *fname, unsigned long fname_len)
#else
// Scilab 6
int CLx_gw_msis00(char *fname, void *pvApiCtx)
#endif
{
  int nb; 
  int *year = NULL; 
  int *doy = NULL; 
  double *sec = NULL; 
  double *glon = NULL; 
  double *glat = NULL;
  double *galt = NULL; 
  double *lst = NULL;
  double *f107 = NULL; 
  double *f107A = NULL; 
  double *ap = NULL; 
  int nb_ap; 
  double *d = NULL; 
  double *t = NULL; 
                

  /* Check the number of input argument */
  CheckInputArgument(pvApiCtx, 12, 12);

  /* Check the number of output argument */
  CheckOutputArgument(pvApiCtx, 2, 2);

  // Input arguments
  clx_gw_getInteger(pvApiCtx, 1, &nb, fname);
  clx_gw_getMatrixInteger(pvApiCtx, 2, &year, fname);
  clx_gw_getMatrixInteger(pvApiCtx, 3, &doy, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 4, &sec, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 5, &glon, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 6, &glat, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 7, &galt, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 8, &lst, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 9, &f107, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 10, &f107A, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 11, &ap, fname);
  clx_gw_getInteger(pvApiCtx, 12, &nb_ap, fname);

  d = (double*) MALLOC(9 * nb * sizeof(double));
  t = (double*) MALLOC(2 * nb * sizeof(double));
  
  if (d == NULL || t == NULL) goto end; 
  
  // note: input and out put arrays: order is column by column
  clx_msis00(nb, year, doy, sec, glon, glat, galt, lst, f107, f107A, ap, nb_ap, d, t);
                
  // Output arguments
  clx_gw_setMatrixDouble(pvApiCtx, 1, d, 9, nb, fname);
  clx_gw_setMatrixDouble(pvApiCtx, 2, t, 2, nb, fname);

  
end: 
  if (d != NULL) FREE(d);
  if (t != NULL) FREE(t);

  return 0;
}
} /* extern "C" */
/* ==================================================================== */


