//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

extern "C"
{
/* ==================================================================== */
#include "api_scilab.h"
#include "Scierror.h"
#include "version.h"
#if SCI_VERSION_MAJOR < 6
// Scilab 5
#include "MALLOC.h"
#else
// Scilab 6
#include "sci_malloc.h"
#endif
#include <localization.h>
#include "CLx_gw_util.h"
#include "clx_tle.h"

/* ==================================================================== */
#if SCI_VERSION_MAJOR < 6
// Scilab 5
int CLx_gw_tle_sgp4(char *fname, unsigned long fname_len)
#else
// Scilab 6
int CLx_gw_tle_sgp4(char *fname, void *pvApiCtx)
#endif
{
  int opt = 0;
  double *cjd = NULL;
  double *ecc = NULL;
  double *inc = NULL;
  double *argp = NULL;
  double *raan = NULL;
  double *ma = NULL;
  double *mm = NULL;
  double *bstar = NULL;
  int nconst = 0;
  int nopsmode = 0;
  double *delta_t = NULL;
  int nb = 0;
  double *pos = NULL;
  double *vel = NULL;
  double *kep = NULL;
  int *status = NULL;
  int err = 0;

  /* Check the number of input argument */
  CheckInputArgument(pvApiCtx, 13, 13);

  /* Check the number of output argument */
  CheckOutputArgument(pvApiCtx, 5, 5);

  // Input arguments
  clx_gw_getInteger(pvApiCtx, 1, &opt, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 2, &cjd, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 3, &ecc, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 4, &inc, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 5, &argp, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 6, &raan, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 7, &ma, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 8, &mm, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 9, &bstar, fname);
  clx_gw_getInteger(pvApiCtx, 10, &nconst, fname);
  clx_gw_getInteger(pvApiCtx, 11, &nopsmode, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 12, &delta_t, fname);
  clx_gw_getInteger(pvApiCtx, 13, &nb, fname);

  pos = (double*) MALLOC(3 * nb * sizeof(double));
  vel = (double*) MALLOC(3 * nb * sizeof(double));
  kep = (double*) MALLOC(6 * nb * sizeof(double));
  status = (int*) MALLOC(nb * sizeof(int));
  
  if (pos == NULL || vel == NULL || kep == NULL || status == NULL) goto end; 
  
  clx_tle_sgp4(&opt, cjd, ecc, inc, argp, raan, ma, mm, bstar, &nconst, &nopsmode, delta_t, &nb, pos, vel, kep, status, &err);

  // Output arguments
  clx_gw_setMatrixDouble(pvApiCtx, 1, pos, 3, nb, fname);
  clx_gw_setMatrixDouble(pvApiCtx, 2, vel, 3, nb, fname);
  clx_gw_setMatrixDouble(pvApiCtx, 3, kep, 6, nb, fname);
  clx_gw_setMatrixInteger(pvApiCtx, 4, status, 1, nb, fname);
  clx_gw_setInteger(pvApiCtx, 5, err, fname);

end: 
  if (pos != NULL) FREE(pos);
  if (vel != NULL) FREE(vel);
  if (kep != NULL) FREE(kep);
  if (status != NULL) FREE(status);

  return 0;
}
} /* extern "C" */
/* ==================================================================== */


