//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

extern "C"
{
/* ==================================================================== */
#include "api_scilab.h"
#include "Scierror.h"
#include "version.h"
#if SCI_VERSION_MAJOR < 6
// Scilab 5
#include "MALLOC.h"
#else
// Scilab 6
#include "sci_malloc.h"
#endif
#include <localization.h>
#include "CLx_gw_util.h"
#include "clx_tle.h"

/* ==================================================================== */
#if SCI_VERSION_MAJOR < 6
// Scilab 5
int CLx_gw_tle_unkozai(char *fname, unsigned long fname_len)
#else
// Scilab 6
int CLx_gw_tle_unkozai(char *fname, void *pvApiCtx)
#endif
{
  double *mm = NULL;
  double *ecc = NULL;
  double *inc = NULL;
  int nconst = 0;
  int nb = 0; 
  double *sma_brw = NULL; 
  double *mm_brw = NULL; 
  int err = 0; 

  /* Check the number of input arguments */
  CheckInputArgument(pvApiCtx, 5, 5);

  /* Check the number of output arguments */
  CheckOutputArgument(pvApiCtx, 3, 3);

  // Input arguments
  clx_gw_getMatrixDouble(pvApiCtx, 1, &mm, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 2, &ecc, fname);
  clx_gw_getMatrixDouble(pvApiCtx, 3, &inc, fname);
  clx_gw_getInteger(pvApiCtx, 4, &nconst, fname);
  clx_gw_getInteger(pvApiCtx, 5, &nb, fname);

  // Call
  sma_brw = (double*) MALLOC(nb * sizeof(double));
  mm_brw = (double*) MALLOC(nb * sizeof(double));

  if (sma_brw == NULL || mm_brw == NULL) goto end; 
  
  clx_tle_unkozai(mm, ecc, inc, &nconst, &nb, 
                  sma_brw, mm_brw, &err); 

  // Output arguments
  clx_gw_setMatrixDouble(pvApiCtx, 1, sma_brw, 1, nb, fname); 
  clx_gw_setMatrixDouble(pvApiCtx, 2, mm_brw, 1, nb, fname);
  clx_gw_setInteger(pvApiCtx, 3, err, fname); 

end: 
  if (sma_brw != NULL) FREE(sma_brw);
  if (mm_brw != NULL) FREE(mm_brw);

  return 0;
}
} /* extern "C" */
/* ==================================================================== */
