//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// ========================
// Test CLx_stela_convert
//
// Tests:
// mean => osculating
// osculating => mean
//
// Nota: reference data obtained using STELA GUI (see file "procedure.txt")
// ========================

TEST_OK = [];

tlbx_path = CLx_home();
exec(fullfile(tlbx_path, "tests", "unit_tests", "CLx_create_extrapolation_inputdata.sci"));

//==================
// Test mean => osc
//==================

x_mean = [24678.105496184424*1000 0.715994110819317 0.044380076018184916 0.0871953024197526 -0.00555460551584653 147.76922492395448*%pi/180]';

// Convert to osculating elements
x_osc = CLx_stela_convert("mean2osc", cjd0 + delta_t($)/86400, x_mean, params, ut1_tref, tt_tref); 

// Expected osculating elements (type == cireq)
expected_xosc   = [24673.648650853327*1000 0.7159145418044449 0.044573784859891406 0.08719417306750195 -0.00557958673924267 147.78483975046908*%pi/180]';


epsilon = 1.e-11;
[TEST_OK($+1),err] = assert_checkalmostequal(x_osc, expected_xosc, epsilon, epsilon);

//==================
// Test osc => mean
//==================

x_osc   = [24673.648650853327*1000 0.7159145418044449 0.044573784859891406 0.08719417306750195 -0.00557958673924267 147.78483975046908*%pi/180]';

// Convert to mean elements
x_mean = CLx_stela_convert("osc2mean", cjd0 + delta_t($)/86400, x_osc, params, ut1_tref, tt_tref); 

// Expected mean elements (type == cireq)
expected_xmean = [24678.105496184424*1000 0.715994110819317 0.044380076018184916 0.0871953024197526 -0.00555460551584653 147.76922492395448*%pi/180]';

epsilon = 1.e-11;
[TEST_OK($+1),err] = assert_checkalmostequal(x_mean, expected_xmean, epsilon, epsilon);
