//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// ========================
// Test CLx_stela_deriv
//
// Tests:
// derivative of state vector at cjd0 + delta_t(:,$)
//
// Nota: reference derivatives obtained using finite differences and Lagrange interpolation (see file "procedure.txt")
// ========================

TEST_OK = [];

tlbx_path = CLx_home();
exec(fullfile(tlbx_path, "tests", "unit_tests", "CLx_create_extrapolation_inputdata.sci"));

// Dates cjd (TREF) for propagation
delta_t = (0 : 10) * 86400;

// Extrapolation
[x, xosc, dxd0, info] = CLx_stela_extrap(cjd0, x0, delta_t, params, res=0, ut1_tref, tt_tref);

// Compute derivatives using CLx  
[dxdt, info] = CLx_stela_deriv(cjd0 + delta_t/86400, x, params, res=0, ut1_tref, tt_tref);


// Expected value at delta_t = 10 (days) = last column 
dxdt11_expected = [
-3.988196455198828e-05
-4.126931689280668e-09
5.225329121172702e-08
-2.911396167101557e-10
-6.442275096851301e-09
0.0001629719141198481
];  


// Old second argument [6,4]: test failed (=> changed to [6,1])
[TEST_OK($+1),err] = assert_checkequal(size(dxdt), [6, 11]); 
epsilon = 1.E-6; // relative error 
[TEST_OK($+1),err] = assert_checkalmostequal(dxdt(:,11), dxdt11_expected, epsilon, 0, "element");
