import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.xml.DOMConfigurator;

import fr.cnes.los.stela.commons.logic.date.DateTime;
import fr.cnes.los.stela.commons.logic.date.StelaDate;
import fr.cnes.los.stela.commons.logic.exception.CommonException;
import fr.cnes.los.stela.commons.logic.log.service.LoggerAgentFactory;
import fr.cnes.los.stela.commons.logic.log.service.LoggerException;
import fr.cnes.los.stela.commons.logic.message.CommonMessage;
import fr.cnes.los.stela.commons.model.AdvancedParameters;
import fr.cnes.los.stela.commons.model.DefaultValues;
import fr.cnes.los.stela.commons.model.Prop;
import fr.cnes.los.stela.commons.model.ShortPeriodsSettings;
import fr.cnes.los.stela.commons.model.atmosmodel.IAtmosphericModel;
import fr.cnes.los.stela.commons.model.dragcoeff.IDragCoef;
import fr.cnes.los.stela.commons.model.ephemeris.Bulletin;
import fr.cnes.los.stela.commons.model.ephemeris.FrameTypes;
import fr.cnes.los.stela.commons.model.ephemeris.NatureTypes;
import fr.cnes.los.stela.commons.model.ephemeris.SimulationType;
import fr.cnes.los.stela.commons.model.ephemeris.type.Type0PosVel;
import fr.cnes.los.stela.commons.model.ephemeris.type.Type2PosVel;
import fr.cnes.los.stela.commons.model.ephemeris.type.Type8PosVel;
import fr.cnes.los.stela.commons.model.solaractivity.ISolarActivity;
import fr.cnes.los.stela.commons.model.spaceobject.ISpaceObject;
import fr.cnes.los.stela.commons.model.spaceobject.SpaceObject;
import fr.cnes.los.stela.commons.model.thirdbody.AbstractThirdBody;
import fr.cnes.los.stela.commons.model.thirdbody.IThirdBody;
import fr.cnes.los.stela.commons.model.thirdbody.ThirdBodyType;
import fr.cnes.los.stela.elib.business.framework.ephemeris.model.EphemerisManager;
import fr.cnes.los.stela.elib.business.framework.simulation.IExtrapolationFeedback;
import fr.cnes.los.stela.elib.business.implementation.atmosmodel.msis00.MSIS00Adapter;
import fr.cnes.los.stela.elib.business.implementation.atmosphericdrag.AtmosphericDragAcc;
import fr.cnes.los.stela.elib.business.implementation.diffeq.DiffEq;
import fr.cnes.los.stela.elib.business.implementation.diffeq.IForceModel;
import fr.cnes.los.stela.elib.business.implementation.dragcoef.constant.model.ConstantDragCoef;
import fr.cnes.los.stela.elib.business.implementation.dragcoef.variable.model.VariableDragCoef;
import fr.cnes.los.stela.elib.business.implementation.earthpotential.tesseral.TesseralAcc;
import fr.cnes.los.stela.elib.business.implementation.earthpotential.zonal.ZonalAcc;
import fr.cnes.los.stela.elib.business.implementation.integrator.Integrator;
import fr.cnes.los.stela.elib.business.implementation.noninertial.ApparentAcc;
import fr.cnes.los.stela.elib.business.implementation.solaractivity.constantmodel.ConstantSolarActivity;
import fr.cnes.los.stela.elib.business.implementation.solaractivity.variablemodel.VariableSolarActivity;
import fr.cnes.los.stela.elib.business.implementation.solarradiationpressure.SRPAcc;
import fr.cnes.los.stela.elib.business.implementation.thirdbody.ThirdBodyAcc;
import fr.cnes.los.stela.elib.business.implementation.thirdbody.moon.model.MoonOrbit;
import fr.cnes.los.stela.elib.business.implementation.thirdbody.sun.model.SunOrbit;
import fr.cnes.los.stela.elib.business.implementation.tide.SolidTidesAcc;
import fr.cnes.los.stela.elib.business.logic.ElibException;
import fr.cnes.los.stela.etoo.business.implementation.simulation.geosimulation.GEOSimulation;
import fr.cnes.los.stela.etoo.business.implementation.simulation.gtosimulation.GTOSimulation;
import fr.cnes.los.stela.processing.flightdynamics.BulletinNatureConverter;
import fr.cnes.los.stela.processing.logic.exception.ProcessingException;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.xml.DOMConfigurator;

import fr.cnes.los.stela.commons.logic.date.DateTime;
import fr.cnes.los.stela.commons.logic.date.StelaDate;
import fr.cnes.los.stela.commons.logic.exception.CommonException;
import fr.cnes.los.stela.commons.logic.log.service.LoggerAgentFactory;
import fr.cnes.los.stela.commons.logic.log.service.LoggerException;
import fr.cnes.los.stela.commons.logic.message.CommonMessage;
import fr.cnes.los.stela.commons.model.AdvancedParameters;
import fr.cnes.los.stela.commons.model.DefaultValues;
import fr.cnes.los.stela.commons.model.Prop;
import fr.cnes.los.stela.commons.model.ShortPeriodsSettings;
import fr.cnes.los.stela.commons.model.atmosmodel.IAtmosphericModel;
import fr.cnes.los.stela.commons.model.dragcoeff.IDragCoef;
import fr.cnes.los.stela.commons.model.ephemeris.Bulletin;
import fr.cnes.los.stela.commons.model.ephemeris.FrameTypes;
import fr.cnes.los.stela.commons.model.ephemeris.NatureTypes;
import fr.cnes.los.stela.commons.model.ephemeris.SimulationType;
import fr.cnes.los.stela.commons.model.ephemeris.type.Type0PosVel;
import fr.cnes.los.stela.commons.model.ephemeris.type.Type2PosVel;
import fr.cnes.los.stela.commons.model.ephemeris.type.Type8PosVel;
import fr.cnes.los.stela.commons.model.solaractivity.ISolarActivity;
import fr.cnes.los.stela.commons.model.spaceobject.ISpaceObject;
import fr.cnes.los.stela.commons.model.spaceobject.SpaceObject;
import fr.cnes.los.stela.commons.model.thirdbody.IThirdBody;
import fr.cnes.los.stela.elib.business.framework.ephemeris.model.EphemerisManager;
import fr.cnes.los.stela.elib.business.framework.simulation.IExtrapolationFeedback;
import fr.cnes.los.stela.elib.business.implementation.atmosmodel.msis00.MSIS00Adapter;
import fr.cnes.los.stela.elib.business.implementation.atmosphericdrag.AtmosphericDragAcc;
import fr.cnes.los.stela.elib.business.implementation.diffeq.DiffEq;
import fr.cnes.los.stela.elib.business.implementation.diffeq.IForceModel;
import fr.cnes.los.stela.elib.business.implementation.dragcoef.constant.model.ConstantDragCoef;
import fr.cnes.los.stela.elib.business.implementation.dragcoef.variable.model.VariableDragCoef;
import fr.cnes.los.stela.elib.business.implementation.earthpotential.tesseral.TesseralAcc;
import fr.cnes.los.stela.elib.business.implementation.earthpotential.zonal.ZonalAcc;
import fr.cnes.los.stela.elib.business.implementation.integrator.Integrator;
import fr.cnes.los.stela.elib.business.implementation.noninertial.ApparentAcc;
import fr.cnes.los.stela.elib.business.implementation.solaractivity.constantmodel.ConstantSolarActivity;
import fr.cnes.los.stela.elib.business.implementation.solaractivity.variablemodel.VariableSolarActivity;
import fr.cnes.los.stela.elib.business.implementation.solarradiationpressure.SRPAcc;
import fr.cnes.los.stela.elib.business.implementation.thirdbody.ThirdBodyAcc;
import fr.cnes.los.stela.elib.business.implementation.thirdbody.moon.model.MoonOrbit;
import fr.cnes.los.stela.elib.business.implementation.thirdbody.sun.model.SunOrbit;
import fr.cnes.los.stela.elib.business.implementation.tide.SolidTidesAcc;
import fr.cnes.los.stela.elib.business.logic.ElibException;
import fr.cnes.los.stela.etoo.business.framework.simulation.SimulationManager;
import fr.cnes.los.stela.etoo.business.implementation.simulation.geosimulation.GEOSimulation;
import fr.cnes.los.stela.etoo.business.implementation.simulation.gtosimulation.GTOSimulation;
import fr.cnes.los.stela.processing.flightdynamics.BulletinNatureConverter;
import fr.cnes.los.stela.processing.logic.exception.ProcessingException;

/**
* Examples of STELA library use: two GTO propagations, a bulletin conversion, and a GEO propagation.<br>
* WARNING: The user has to define the root directory before launching the main using {@link Prop#defineROOT(String)}.
* @author Fabrice MOULIN-NEXON
* @author Emmanuel Bignon
* @author Romain Pinede
*/
public class Example {

    /**
     * Main example.
     * @param args arguments (not needed here)
     * @throws ProcessingException 
     * @throws ElibException 
     * @throws CommonException 
     */
    public static void main(final String[] args) throws CommonException, ElibException, ProcessingException {

        // ===================== INITIALIZATION ROOT DIRECTORY =====================

        // Set root directory: this is mandatory (default root directory is your working directory)
        Prop.defineROOT("C:\\RootDirectory..."); // Replace value by the path where you set up STELA

        // Other file definitions (optional):
        // Prop allows you to define several files with a relative path (referring to ROOT) or absolute path
        // Example 1: Prop.setStelaSolarActivity("mySolarActivityFile.txt"); // Solar activity file is Prop.ROOT + mySolarActivityFile.txt
        // Example 2: Prop.setStelaSolarActivity("C:\Documents\STELA\mySolarActivityFile.txt"); // Solar activity file is C:\Documents\STELA\mySolarActivityFile.txt
        
        // ============================== INITIALIZATION LOGGER ==============================

        System.out.println("Start STELA library mode!");

        // Configure a basic console logger for log4j that will show everything logged

        // Get file in stela_console_log4j.xml in stela-commons-X.X.X.jar archive
        // Add the file in resource folder of your project
        // Modify level info as   
        //        <root>
        //            <level value="info" />
        //            <!-- During development, replace null by console -->
        //            <appender-ref ref="console" />
        //        </root>
        DOMConfigurator.configure(ClassLoader.getSystemResource("stela_console_log4j.xml"));

        // ===================== INITIALIZATION LANGUAGE =====================
        // Set language: this is optionnal (default language is english)
        Prop.defineLANG("en");

        try {
            LoggerAgentFactory.getLoggerAgent().init("stela_service_log_config.xml");
            LoggerAgentFactory.getLoggerAgent().newLogger("StelaMainLog");
        } catch (final LoggerException e) {
            System.err.println("Error during initialize logger");
            System.exit(2);
        }

        final fr.cnes.los.stela.commons.logic.log.service.Logger logger = LoggerAgentFactory.getLoggerAgent().getDefaultLogger();

        logger.log(CommonMessage.initialization());

        // ============================== LAUNCH SIMULATION ==============================
        simpleGTOPropagation();
        
        complexGTOPropagation();
        
        meanToOscConversion();
        
        GEOIterativePropagation();
        
        System.out.println("End STELA library mode!");
    }
    
    /**
     * Example of a simple GTO propagation using the ISimulation.simulate() method.
     */
    private static void simpleGTOPropagation() throws ElibException, ProcessingException, CommonException {
        // Initial date (TAI scale)
        final StelaDate initialDate = new StelaDate(DateTime.parseDate("2015-08-15T23:00:00.000"));

        // Initial orbit and state
        double za = 3.552E7; // apogee altitude
        double zp = 180000.0; // perigee altitude
        double i = 0.17453292519943295; // inclination
        double raan = 0; // right ascension of ascending node
        double aop = 0; // argument of periapsis
        double M = 0; // mean anomaly
        Type0PosVel initialOrbit = new Type0PosVel(zp, za, i, aop, raan, M, NatureTypes.MEAN, FrameTypes.CIRF);
        Bulletin initialState = new Bulletin(initialDate, initialOrbit);

        // Spacecraft
        double mass = 1000.; // kg
        double dragArea = 10.; // m2
        double reflectivityArea = 10.; // m2
        double reflectivityCoef = 1.5;
        IDragCoef dragCoef = new VariableDragCoef();
        ISpaceObject spaceObject = new SpaceObject("My spacecraft", mass, dragArea, reflectivityArea, reflectivityCoef, dragCoef);

        // Simulation initialization
        double simulationDuration = 100; // Years
        ISolarActivity solarActivity = new VariableSolarActivity();
        IAtmosphericModel atmosModel = new MSIS00Adapter();
        EphemerisManager ephManager = new EphemerisManager(initialState);
        double integrationStep = 86400; // s
        double ephStep = 86400; // s (should be a multiple of the integration step)
        int dragQuadPoints = 33;
        int srpQuadPoints = 11;
        int atmosDragRecomputeStep = 1;
        boolean dragSwitch = true;
        boolean sunSwitch = true;
        boolean moonSwitch = true;
        boolean srpSwitch = true;
        boolean zonalSwitch = true;
        boolean tesseralSwitch = true;
        boolean solidTidesSwitch = true;
        double ttMinusUT1 = 68.184;
        double reentryAltitude = 80000; // m
        double nbIntegrationStepTesseral = 5.;
        int tesseralOrder = 7;
        int zonalOrder = 7;

        GTOSimulation gtoSim = new GTOSimulation("Default Author", "Propagation example", spaceObject, simulationDuration, ephManager,
                ephStep, integrationStep, ttMinusUT1, reentryAltitude, zonalSwitch, zonalOrder, tesseralSwitch, tesseralOrder,
                nbIntegrationStepTesseral, sunSwitch, moonSwitch, dragSwitch, atmosModel, solarActivity, dragQuadPoints, atmosDragRecomputeStep,
                srpSwitch, srpQuadPoints, solidTidesSwitch);

        boolean extrapolationStatus = gtoSim.simulate(null);

        // Get ephemeris
        ArrayList<Bulletin> ephemeris = ephManager.getEphemerisList();

        // Get the parameters of the orbit at the end of the propagation
        Bulletin finalState = ephemeris.get(ephemeris.size() - 1);
        // The final orbit parameters can be retrieved in type 8
        Type8PosVel finalOrbitType8 = (Type8PosVel) finalState.getPosVel();
        // Convert the final orbit into a Keplerian orbit
        Type2PosVel finalOrbitType2 = finalOrbitType8.toType2();
    }

    /**
     * Example of a GTO propagation using more complex settings.
     */
    private static void complexGTOPropagation() throws CommonException, ElibException, ProcessingException {
        // Initial date (TAI scale)
        StelaDate initialDate = new StelaDate(DateTime.parseDate("2015-08-15T23:00:00.000"));

        // Initial orbit and state
        double za = 3.552E7; // apogee altitude (m)
        double zp = 180000.0; // perigee altitude (m)
        double i = 0.17453292519943295; // inclination (rad)
        double raan = 0; // right ascension of ascending node (rad)
        double aop = 0; // argument of periapsis (rad)
        double M = 0; // mean anomaly (rad)
        Type0PosVel initialOrbit = new Type0PosVel(zp, za, i, aop, raan, M, NatureTypes.MEAN, FrameTypes.CIRF);
        Bulletin initialState = new Bulletin(initialDate, initialOrbit);

        // Spacecraft
        double mass = 1000.; // kg
        double dragArea = 10.; // m2
        double reflectivityArea = 10.; // m2
        double reflectivityCoef = 1.5;
        double Cx = 2.2;
        IDragCoef dragCoef = new ConstantDragCoef(Cx);
        ISpaceObject spaceObject = new SpaceObject("My spacecraft", mass, dragArea, reflectivityArea, reflectivityCoef, dragCoef);

        // Forces initialization
        ISolarActivity solarActivity = new ConstantSolarActivity();
        solarActivity.setConstantF107(145);
        solarActivity.setConstantAP(9);
        IAtmosphericModel atmosModel = new MSIS00Adapter();
        int dragQuadPoints = 33;
        int srpQuadPoints = 11;

        // Forces (without tesseral perturbations in our case)
        // Sun and moon with initial date
        IThirdBody sun = new SunOrbit(DefaultValues.TT_MINUS_UT1);
        IThirdBody moon = new MoonOrbit(DefaultValues.TT_MINUS_UT1);

        ZonalAcc zonalAcc = new ZonalAcc(DefaultValues.GTO_ZONAL_ORDER, true); // true is to take J22 contribution into account
        TesseralAcc tesseralAcc = null;
        ThirdBodyAcc sunAcc = new ThirdBodyAcc(AdvancedParameters.THIRD_BODY_POT_DEGREE, sun);
        ThirdBodyAcc moonAcc = new ThirdBodyAcc(AdvancedParameters.THIRD_BODY_POT_DEGREE, moon);
        AtmosphericDragAcc dragAcc = new AtmosphericDragAcc(spaceObject, dragQuadPoints, atmosModel, solarActivity,
                AdvancedParameters.Z_LIMIT_ATMOS, sun, moon);
        SRPAcc srpAcc = new SRPAcc(spaceObject, srpQuadPoints, true, sun); // true is to compute SRP with eclipses
        ApparentAcc apparentAcc = new ApparentAcc(AdvancedParameters.COMPL_ACC_QUADRATURE_POINTS, FrameTypes.CIRF,
                AdvancedParameters.REFERENCE_FRAME, DefaultValues.TT_MINUS_UT1);
        SolidTidesAcc solidTidesAcc = new SolidTidesAcc(true, true, sun, moon); // 1st true for the sun contribution, 2nd true for the moon

        // The differential equation can be used to manually compute contributions
        List<IForceModel> forces = new ArrayList<IForceModel>();
        forces.add(zonalAcc);
        forces.add(tesseralAcc);
        forces.add(sunAcc);
        forces.add(moonAcc);
        forces.add(dragAcc);
        forces.add(srpAcc);
        forces.add(apparentAcc);
        forces.add(solidTidesAcc);
        DiffEq diffEq = new DiffEq(initialState, forces);

        // Integrator initialization
        EphemerisManager ephManager = new EphemerisManager(initialState);
        double simulationDuration = 100; // years
        double integrationStep = 86400.0; // s
        double ephInterval = 86400.0; // ephemeris interval (s) - should be a multiple of the integration step
        int paramUpdatePeriod = 1; // Drag computing period (in number of integration steps)
        double reentryAltitude = 80000; // meters
        IExtrapolationFeedback feedee = null; // This parameter is used by the progression bar in the GUI
        // Short periods settings : zonal contribution (true/false), zonal order, sun contribution, moon contribution
        ShortPeriodsSettings shortPeriodsSettings = new ShortPeriodsSettings(true, 7, true, true);
        boolean timeDerivativesFlag = false;

        // Define the integrator and perform the integration
        // (the switch corresponding to the false value is used by the high-level Simulation when iterative mode is activated)
        Integrator integrator = new Integrator("RK6", SimulationType.GTO, ephManager, diffEq, initialState, integrationStep,
                simulationDuration, ephInterval, paramUpdatePeriod, reentryAltitude, feedee, DefaultValues.TT_MINUS_UT1,
                shortPeriodsSettings, timeDerivativesFlag, false);
        integrator.integrate();

        // Once the integration is complete, get the ephemeris
        ArrayList<Bulletin> records = ephManager.getEphemerisList();
    }

    /**
     * Example of conversion from MEAN to OSCULATING orbital elements.
     * Be carreful to update Sun and Moon position before converting bulletin.
     */
    private static void meanToOscConversion() throws CommonException, ProcessingException {

        // Define date (TAI scale) and orbital parameters
        StelaDate initialDate = new StelaDate(DateTime.parseDate("2015-08-15T23:00:00.000"));
        double za = 3.552E7; // apogee altitude (m)
        double zp = 180000.0; // perigee altitude (m)
        double i = 0.17453292519943295; // inclination (rad)
        double raan = 0; // right ascension of ascending node (rad)
        double aop = 0; // argument of periapsis (rad)
        double M = 0; // mean anomaly (rad)
        Type0PosVel initialOrbit = new Type0PosVel(zp, za, i, aop, raan, M, NatureTypes.MEAN, FrameTypes.CIRF);
        Bulletin initialState = new Bulletin(initialDate, initialOrbit);

        double ttMinusUT1 = 67.184;
        boolean zonalSwitch = true;
        boolean sunSwitch = true;
        boolean moonSwitch = true;
        int zonalOrder = 7;

        // Initialize Moon and Sun
        IThirdBody moon = new MoonOrbit(ttMinusUT1);
        IThirdBody sun = new SunOrbit(ttMinusUT1);

        // Convert bulletin to osculating parameters
        ShortPeriodsSettings shortPeriodsSettings = new ShortPeriodsSettings(zonalSwitch, zonalOrder, sunSwitch, moonSwitch);
        BulletinNatureConverter.convertToOsculating(initialState, moon, sun, shortPeriodsSettings);

        // Get the osculating parameters (keplerian)
        Type0PosVel finalOrbitType8 = (Type0PosVel) initialState.getPosVel();
        Type2PosVel finalOrbitType2 = finalOrbitType8.toType2();
    }

    /**
     * Example of a GEO propagation with some additions (iterative mode).
     */
    private static void GEOIterativePropagation() throws ElibException, ProcessingException, CommonException {
        // Initial date (TAI scale)
        StelaDate initialDate = new StelaDate(DateTime.parseDate("1998-01-01T00:00:00.000"));

        // Initial orbit and state
        double a = 42404000; // semi-major axis (m)
        double ex = 0.00000999553; // first component of eccentricity vector
        double ey = 0.00001000447; // second component of eccentricity vector
        double ix = -0.00002297372; // first component of inclination vector
        double iy = 0.00000412259; // second component of inclination vector
        double lambdaEq = 1.4218432997029051; // equatorial longitude (rad)
        Type8PosVel initialOrbit = new Type8PosVel(a, lambdaEq, ex, ey, ix, iy, NatureTypes.MEAN, FrameTypes.CIRF);
        Bulletin initialState = new Bulletin(initialDate, initialOrbit);

        // Spacecraft
        double mass = 2000.; // kg
        double dragArea = 20.; // m2
        double reflectivityArea = 20.; // m2
        double reflectivityCoef = 1.5;
        double Cx = 2.2;
        IDragCoef dragCoef = new ConstantDragCoef(Cx);
        ISpaceObject spaceObject = new SpaceObject("My spacecraft", mass, dragArea, reflectivityArea, reflectivityCoef, dragCoef);

        // Simulation initialization
        double simulationDuration = 100; // Years
        ISolarActivity solarActivity = new VariableSolarActivity();
        IAtmosphericModel atmosModel = new MSIS00Adapter();
        EphemerisManager ephManager = new EphemerisManager(initialState);
        double ephStep = 86400; // s
        double integrationStep = 86400; // s
        int dragQuadPoints = 33;
        int srpQuadPoints = 11;
        int atmosDragRecomputeStep = 1;
        boolean dragSwitch = false;
        boolean sunSwitch = true;
        boolean moonSwitch = true;
        boolean srpSwitch = true;
        boolean zonalSwitch = true;
        boolean tesseralSwitch = true;
        boolean solidTidesSwitch = true;
        double ttMinusUT1 = 67.184;
        double reentryAltitude = 80000; // m
        double nbIntegrationStepTesseral = 5.;
        int tesseralOrder = 7;
        int zonalOrder = 7;

        // Settings for GEO iterative mode
        double exclusionDuration = 100.; // Duration needed away from the GEO protected region
        double expFinalMinAlt = 0.; // Expected final minimum altitude
        double targetEccentricityX = 0.; // Target projection on x of the eccentricity vector
        double targetEccentricityY = 0.; // Target projection on y of the eccentricity vector
        double funcValueAccuracy = 0.; // Required accuracy on the iteration solving function

        GEOSimulation geoSim = new GEOSimulation("Default Author", "Propagation example", spaceObject, simulationDuration, ephManager,
                ephStep, integrationStep, ttMinusUT1, reentryAltitude, zonalSwitch, zonalOrder, tesseralSwitch, tesseralOrder,
                nbIntegrationStepTesseral, sunSwitch, moonSwitch, dragSwitch, atmosModel, solarActivity, dragQuadPoints, atmosDragRecomputeStep,
                srpSwitch, srpQuadPoints, solidTidesSwitch, exclusionDuration, expFinalMinAlt, funcValueAccuracy, targetEccentricityX,
                targetEccentricityY);

        // Allow unitary output of derivatives
        geoSim.setTimeDerivativesFlag(true);

        try {
            // Launch extrapolation
            if (!geoSim.simulate(null)) {

                // Extrapolation succeeded

                // Display some informations
                System.out.println(geoSim.getInformations());

                // Display 10 first time derivatives
                System.out.println(geoSim.getEphemerisManager().getDiffEqDataList().subList(0, 9));

                // Save simulation
                SimulationManager sm = new SimulationManager();
                try {
                    sm.save("C:\\example_GEO_sim.xml", geoSim);
                } catch (CommonException e) {
                    System.err.println("Error during saving simulation");
                    System.exit(2);
                }
            } else {
                // Extrapolation failed
                System.err.println("Extrapolation failed");
            }
        } catch (ElibException e) {
            // Validity control failed
            System.out.println("Validity control failed");
        }

        // Get ephemeris
        ArrayList<Bulletin> ephemeris = ephManager.getEphemerisList();

        // Get the parameters of the orbit at the end of the propagation
        Bulletin finalState = ephemeris.get(ephemeris.size() - 1);
        // The final orbit parameters can be retrieved in type 8
        Type8PosVel finalOrbitType8 = (Type8PosVel) finalState.getPosVel();
    }
}