#!/usr/bin/python

####
## STELA batch mode Python example
## Tested on Linux, Windows and Sun
##
## The STELA simulation "example_GEO_sim.xml" is loaded and extrapolated,
## then the semi major axis value is decreased by one kilometer before another extrapolation.
## The value is modified in a temporary xml file.
## You can activate the DEBUG_MODE with the flag DEBUG_MODE
## You can also save intermediate xml file. Uncomment line in the main.
## It is not recommended for novice users to modify METHODS part.
## For more details, see User manual.
####
import sys,os
import tempfile
import platform
import datetime


### INTERNAL VARIABLE. Do not change unless you know what you are doing
__temporaryfile = tempfile.NamedTemporaryFile().name + "stela_tmp_sim.xml"
__workingfile = tempfile.NamedTemporaryFile().name + "stela_work_sim.xml"

# when this flag is enable, debug mode is activated
DEBUG_MODE=False

# STELA_ROOT set during install. 
STELA_ROOT='$INSTALL_PATH'

######################################## METHODS ########################################

# debug if DEBUG_MODE is true
def debug(mess):
    if DEBUG_MODE:
        print "DEBUG# " + mess
    #END IF
#END debug

#copy file src to dest
def copy(src,dest):
    debug("copy " + src + " to " + dest)
    
    try:
        #Open file for reading
        s=open(src,"r" )
    except:
        end(10, "File <" + src +"> not found")
    try:
        #Open file for writing
        t=open(dest,"w+" )
    except:
        end(20, "File <" + dest +" > not found")
    
    while 1:
       # Read 1 buffer of file
       buf=s.read(1)
       if buf:
          t.write(buf)
       else:
          break
    #END WHILE
    #close file
    s.close()
    t.close()
#END copy

#Move extrapolation file src to dest
def moveExtrapolationFile(src,dest):
    #remove dest file
    remove_file(dest)
    os.rename(src,dest)
    remove_file( dest.replace(".xml",".txt"))
    os.rename(src.replace(".xml",".txt"), dest.replace(".xml",".txt"))
#END moveExtrapolationFile

#Remove filename
def remove_file(filename):
    if os.path.isfile(filename):
        os.remove(filename)
    
    if os.path.isfile(filename):
        end(30, "Error when remove file <" + filename +">")
#END remove_file

#End prog
def end(CR, mess):
    print "END PROG (CR=" + str(CR) + ") # " + mess
    raw_input('Press enter to finish the program...')
    sys.exit(CR)
#END end 
    
#get a value
# -param key : key in the xml file
def getValue( key):
     from xml.dom.minidom import parse
     
     doc = parse(__workingfile)
     node = doc.getElementsByTagName(key)
     try:
         tmp = float(node[0].firstChild.nodeValue)
     except:
         end(40, "Key <" + key + "> not found")
         
     return tmp 
 #END getValue

#get a value in the final state
# -param key : key in the xml file
def getFinalStateValue(key):
     from xml.dom.minidom import parse
     
     doc = parse(__workingfile)
     node = doc.getElementsByTagName("finalState")[0].getElementsByTagName(key)

     try:
         tmp = float(node[0].firstChild.nodeValue)
     except:
         end(50, "Key <" + key + "> not found in final bulletin")
     
     return tmp
#END getFinalStateValue

#define a value in the working xml file :: __workingfile
def setValue(key, value):
    from xml.dom.minidom import parse
    
    # create a backup of original file
    remove_file(__temporaryfile)    
    
    if not os.path.isfile(__workingfile):
        end(60, "File <"+ __workingfile +"> not found")
    
    try:
        # change text value of element
        doc = parse(__workingfile)
        node = doc.getElementsByTagName(key)
        node[0].firstChild.nodeValue = str(value)
    except:
        end(70, "Key <" + key +"> not found")

    os.rename(__workingfile,__temporaryfile)
    if not os.path.isfile(__temporaryfile):   
        end(80, "Can't write <"+ __temporaryfile +"> ")

    # persist changes to new file
    xml_file = open(__workingfile, "w")
    doc.writexml(xml_file, encoding="ISO-8859-1")
    xml_file.close()
    if not os.path.isfile(__workingfile):
        end(90, "Can't write final file")

#FIN setValue

#Chose the good prog between unix and windows
# -no-params    
def choseprog():
    platf=platform.system()
    if platf == 'Linux':
        cmd = os.path.join(STELA_ROOT,"bin","stela-batch.sh")
    elif platf == 'Windows':
        cmd = os.path.join(STELA_ROOT,"bin","stela-batch.bat")
    else:
        end(100, "unknown OS")
    #END IF
    debug("PROG : " + cmd)
    
    return cmd
#FIN choseprog

#Launch an extrapolation with workingfile
# -no-params
def extrapolate():
    import subprocess
    prog=choseprog()
    
    option=" -i " + __workingfile + " -o " + __temporaryfile

    os.chdir(os.path.dirname(os.path.normpath(prog)))

    platf=platform.system()
    if platf == 'Linux':
            CR=os.system("./" + os.path.basename(prog) + option  )
    elif platf == 'Windows':
            CR=os.system( os.path.basename(prog) + option  )
    else:
        end(105, "unknown OS")
    #END IF

    if CR != 0:
        end(CR, "Error when launching extrapolation")
    
    #Test file generated after extrapolation
    if not os.path.isfile(__temporaryfile):
        end(110, "Error when write output file during launching extrapolation")
    
    #Remove working file
    remove_file(__workingfile)
    
    moveExtrapolationFile(__temporaryfile, __workingfile)
    if not os.path.isfile(__workingfile):
        end(120, "Error when move temporary file in working file")
#FIN extrapolate

#Send message in a logfile, they are automatically separeted by ';'
# -param1 filename of logfile
# -param* value 
def logfile(filename, *mess):
    f = open(filename, 'a')
    separator=';'
    cr='\n'

    tmp=''
    for m in mess:
        if tmp == '':
            tmp = str(m)
        else:
            tmp = tmp + separator + str(m)
    debug("IN " + filename + " > " + tmp)
    f.write(tmp + cr)
    f.close()
#END logfile
  
######################################## MAIN ######################################## 

# Input file 
__origfile = os.path.join(STELA_ROOT, "examples", 'example_LEO_sim.xml')
#Log file
__defaultlogfile =  os.path.join(os.path.expanduser("~"), "stela_batch-log_file.csv")
#Initialize logfile
remove_file(__defaultlogfile)

logfile(__defaultlogfile,"# STELA 3.2")
logfile(__defaultlogfile,"# By <Your name> ")
logfile(__defaultlogfile,"# " + datetime.datetime.now().ctime())
logfile(__defaultlogfile,"semiMajorAxis(BULL_IN)","eccentricity(BULL_IN)","inclination(BULL_IN)","rAAN(BULL_IN)","argOfPerigee(BULL_IN)","meanAnomaly(BULL_IN)",
                         "semiMajorAxis(BULL_OUT)","eccentricity(BULL_OUT)","inclination(BULL_OUT)","rAAN(BULL_OUT)","argOfPerigee(BULL_OUT)","meanAnomaly(BULL_OUT)")

#Copy origin file to work
copy(__origfile, __workingfile)

print "=========== START ==========="
for i in range(1, 11):
    print "=========== LOOP " + str(i) + " ==========="
    
    # Uncomment following 2 lines to reduce time execution of script and enable DEBUG_MODE flag
    #if DEBUG_MODE:
    #    setValue("simulationDuration", 10)

    #Launch extrapolation on working file
    extrapolate()
    #Log information in a file
    logfile(__defaultlogfile,getValue("semiMajorAxis"),getValue("eccentricity"),getValue("inclination"),getValue("rAAN"),getValue("argOfPerigee"),getValue("meanAnomaly"),
                             getFinalStateValue("semiMajorAxis"),getFinalStateValue("eccentricity"),getFinalStateValue("inclination"),getFinalStateValue("rAAN"),getFinalStateValue("argOfPerigee"),getFinalStateValue("meanAnomaly"))

    print "initial value of semiMajorAxis : " +  str(getValue("semiMajorAxis")) + " m"
    print "final value of semiMajorAxis   : " +  str(getFinalStateValue("semiMajorAxis")) + " m"

    #change value
    setValue("semiMajorAxis", getValue("semiMajorAxis") - 1000.)

#END FOR

print "=========== END ==========="
print "Results are available in file : " + __defaultlogfile
remove_file(__temporaryfile)

end(0, "End prog")
